SmartyCatManaBar = AceLibrary("AceAddon-2.0"):new("AceDB-2.0", "AceEvent-2.0", "AceConsole-2.0")

local L = AceLibrary("AceLocale-2.2"):new("SmartyCatManaBar")

function SmartyCatManaBar:OnInitialize()
    local _, class = UnitClass("player")
	if class ~= "DRUID" then
		return
	end
	
	self.dewdrop = AceLibrary("Dewdrop-2.0")
	self.SML = AceLibrary("SharedMedia-1.0")
	self:RegisterDB("SmartyCatManaBarDB")
	
	self.SML:Register("statusbar", "Perl", "Interface\\AddOns\\SmartyCatManaBar\\perl")
	self.SML:Register("statusbar", "Charcoal", "Interface\\AddOns\\SmartyCatManaBar\\charcoal")
	self.SML:Register("statusbar", "Otravi", "Interface\\AddOns\\SmartyCatManaBar\\otravi")
	self.SML:Register("statusbar", "Smooth", "Interface\\AddOns\\SmartyCatManaBar\\smooth")
	self.SML:Register("statusbar", "Glaze", "Interface\\AddOns\\SmartyCatManaBar\\glaze")
	self.SML:Register("statusbar", "Granite", "Interface\\AddOns\\SmartyCatManaBar\\granite")
	
	self:RegisterDefaults("profile", {
		border = "Default",
		height = 9,
		width = 130,
		fontsize = 9,
		texture = "Charcoal",
		barcolor = { r = 0, g = 0, b = 1, a=1 },
		bordercolor = { r = .5, g = 0.5, b = 0.5, a=1 },
		backcolor = { r = 0, g = 0, b = 0, a=.5 },
		fontcolor = { r = 1.0, g = 1.0, b = 1.0 },
		hideFrame = false,
		hideInCaster = true,
		hideWhenFull = false,
	})
	
	self:RegisterChatCommand({ "/SmartyCatManaBar", "/scmb"}, {
		type = "execute", name = "SmartyCatManaBar", desc = "SmartyCatManaBar",
		func = function() 
			self:LoadOptions() 
			self.dewdrop:Open(UIParent, 'children', 
				function() self.dewdrop:FeedAceOptionsTable(self.opts) end, 
				'cursorX', true, 'cursorY', true) 
		end,
	})
	
end

function SmartyCatManaBar:OnEnable()
	local _, class = UnitClass("player")
	if class ~= "DRUID" then
		return
	end
	
	if not self.db.profile.hideFrame then
		self:EnableFrame()
	end
end

function SmartyCatManaBar:EnableFrame()
	if not self.frame then
		self:SetupFrame()
	end
	self:RegisterEvent("SmartyCat_ManaUpdated", "UpdateFrame")
	self:RegisterEvent("SharedMedia_SetGlobal", "UpdateLook")
	self:UpdateFrame()
end

function SmartyCatManaBar:HideFrame()
	self.frame:Hide()
	self:UnregisterEvent("SmartyCat_ManaUpdated")
	self:UnregisterEvent("SharedMedia_SetGlobal")
end

function SmartyCatManaBar:UpdateFrame()
	local f = self.frame 
	if (self.db.profile.hideInCaster and UnitPowerType("player") == 0) or (SmartyCat:GetManaValue() == 1 and self.db.profile.hideWhenFull) then
		f:Hide()
	else
		f:Show()
		f.manaText:SetText(SmartyCat:GetCurrMana().."/"..SmartyCat:GetMaxMana())
		f.pText:SetText(SmartyCat:GetManaPercent().."%")
		f.bar:SetValue(SmartyCat:GetManaValue())
	end
end

function SmartyCatManaBar:LoadOptions()
	if self.opts then return end
	
	local db = self.db.profile
	self.opts = {
		type = "group",
		args = {
			header = {
				type = "header",
				name = "SmartyCatManaBar",
				order = 1,
			},
			frame = {
				type = "group",
				name = L["Mana Bar"],
				desc = L["Set Mana Bar"],
				order = 2,
				hidden = function() return db.hideFrame end,
				args = {
					height = {
						type = "range",
						name = L["Height"],
						desc = L["Set the bar height"],
						set = function(v) db.height = v self:UpdateLook() end,
						get = function() return db.height end,
						min = 4, max = 50, step = 1,
						order = 1,
					},
					width = {
						type = "range",
						name = L["Width"],
						desc = L["Set the bar width"],
						set = function(v) db.width = v self:UpdateLook() end,
						get = function() return db.width end,
						min = 100, max = 300, step = 2,
						order = 2,
					},
					fontsize = {
						type = "range",
						name = L["Font Size"],
						desc = L["Set the font size"],
						set = function(v) db.fontsize = v self:UpdateLook() end,
						get = function() return db.fontsize end,
						min = 6, max = 26, step = 1,
						order = 3,
					},
					border = {
						type = "text",
						name = L["Border"],
						desc = L["Set the border style"],
						set = function(v) db.border = v self:UpdateLook() end,
						get = function() return db.border end,
						validate = { "None", "Default", "Textured", },
						usage = nil,
						order = 4,
					},
					bartexture = {
						type = "text", 
						name = L["Bar Texture"],
						desc = L["Set the bar texture"],
						get = function() return db.texture end,
						set = function(v) 
							db.texture = v 
							self.frame.bar:SetStatusBarTexture(self.SML:Fetch("statusbar", db.texture)) 
							self:UpdateLook()
						end,
						validate = self.SML:List("statusbar"),
						usage = nil,
						order = 5,
					},
					barcolor = {
						type = "color",
						name = L["Bar Color"],
						desc = L["Set the bar color"],
						get = function() return db.barcolor.r, db.barcolor.g, db.barcolor.b, db.barcolor.a end,
						set = function(r,g,b,a) 
							db.barcolor = {r=r, g=g, b=b, a=a}
							self.frame.bar:SetStatusBarColor(r, g, b, a) 
						end,
						hasAlpha = true,
						order = 6,
					},
					bordercolor = {
						type = "color",
						name = L["Border Color"],
						desc = L["Set the border color"],
						get = function() return db.bordercolor.r,db.bordercolor.g,db.bordercolor.b,db.bordercolor.a end,
						set = function(r,g,b,a) db.bordercolor = {r=r,g=g,b=b,a=a} self:UpdateLook() end,
						hasAlpha = true,
						order = 7,
					},
					backcolor = {
						type = "color",
						name = L["Background Color"], 
						desc = L["Set the background color"],
						get = function() return db.backcolor.r, db.backcolor.g, db.backcolor.b, db.backcolor.a end,
						set = function(r,g,b,a) db.backcolor = {r=r,g=g,b=b,a=a} self:UpdateLook() end,
						hasAlpha = true,
						order = 8,
					},
					fontcolor = {
						type = "color",
						name = L["Font Color"], 
						desc = L["Set the font color"],
						get = function() return db.fontcolor.r, db.fontcolor.g, db.fontcolor.b end,
						set = function(r,g,b) db.fontcolor = {r=r,g=g,b=b} self:UpdateLook() end,
						order = 9,
					},
				},
			},
			hideInCaster = {
				type = "toggle",
				name = L["Hide in Caster Form"], 
				desc = L["Hides the mana bar when not in Cat or Bear form"],
				get = function() return db.hideInCaster end,
				set = function(v)
					db.hideInCaster = v
					self:UpdateFrame()
				end,
				order = 3,
				hidden = function() return db.hideFrame end,
			},
			hideWhenFull = {
				type = "toggle",
				name = L["Hide at Full Mana"], 
				desc = L["Hides the mana bar when mana is full"],
				get = function() return db.hideWhenFull end,
				set = function(v)
					db.hideWhenFull = v
					self:UpdateFrame()
				end,
				order = 4,
				hidden = function() return db.hideFrame end,
			},
			hideframe = {
				type = "toggle",
				name = L["Hide Mana Bar"], 
				desc = L["Hides the Mana Bar"],
				get = function() return db.hideFrame end,
				set = function(v)
					db.hideFrame = v
					if v then
						self:HideFrame()
					else
						self:EnableFrame()
					end
				end,
				order = 5,
			},
		},
	}
end

function SmartyCatManaBar:SetupFrame()
	local f = CreateFrame("Frame", "SCMBFrame", UIParent)
	local db = self.db.profile
	if not db.x then
		f:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
	else
		f:SetPoint("TOPLEFT", UIParent, "TOPLEFT", db.x, db.y)
	end
	f:SetMovable(true)
	f:EnableMouse(true)
	f:RegisterForDrag("LeftButton")
	f:SetScript("OnDragStart", function() if IsShiftKeyDown() then this:StartMoving() end end)
	f:SetScript("OnDragStop", function() 
		this:StopMovingOrSizing() 
		local a,b,c,d,e = this:GetPoint()
		db.x = floor(d + 0.5)
		db.y = floor(e + 0.5)
	end)
	
	f.bar = CreateFrame("StatusBar", nil, f)
	f.bar:SetMinMaxValues(0, 1)
	f.manaText = f.bar:CreateFontString(nil, "OVERLAY")
	f.pText = f.bar:CreateFontString(nil, "OVERLAY")
	f.bar:ClearAllPoints()
	f.bar:SetPoint("TOPLEFT", 4, -4)
	f.bar:SetPoint("BOTTOMRIGHT", -4, 4)
	
	self.dewdrop:Register(f, 'children', 
		function() self:LoadOptions() self.dewdrop:FeedAceOptionsTable(self.opts) end,
		'cursorX', true, 'cursorY', true)
	
	self.frame = f
	self:UpdateLook()
end

function SmartyCatManaBar:UpdateLook()
	local db = self.db.profile
	local font = GameFontHighlightSmall:GetFont()
	local f = self.frame
	local inset = 4
	
	f:SetWidth(db.width+10)
	f:SetHeight(db.height+10)
	if db.border == "Textured" then inset = 2 end
	f:SetBackdrop({ 
		bgFile = self.SML:Fetch("statusbar", db.texture),
		edgeFile = (db.border=="Default" and "Interface\\Tooltips\\UI-Tooltip-Border") or "", edgeSize = 12, 
		insets = {left = inset, right = inset, top = inset, bottom = inset},
	})
	f:SetBackdropColor(db.backcolor.r, db.backcolor.g, db.backcolor.b, db.backcolor.a)
	f:SetBackdropBorderColor(db.bordercolor.r, db.bordercolor.g, db.bordercolor.b, db.bordercolor.a) 
	
	f.bar:SetStatusBarTexture(self.SML:Fetch("statusbar", db.texture))
	f.bar:SetStatusBarColor(db.barcolor.r, db.barcolor.g, db.barcolor.b, db.barcolor.a)
	
	f.pText:SetFont(font, db.fontsize)
	f.pText:ClearAllPoints()
	f.manaText:SetFont(font, db.fontsize)
	f.manaText:ClearAllPoints()
	f.pText:SetShadowColor(0.1, 0.1, 0.1)
	f.pText:SetShadowOffset(1,-1)
	f.manaText:SetShadowColor(0.1, 0.1, 0.1)
	f.manaText:SetShadowOffset(1,-1)
	f.pText:SetTextColor(db.fontcolor.r, db.fontcolor.g, db.fontcolor.b)
	f.manaText:SetTextColor(db.fontcolor.r, db.fontcolor.g, db.fontcolor.b)
	
	f.pText:SetJustifyH("RIGHT")
	f.pText:SetWidth(f:GetWidth())
	f.pText:SetPoint("RIGHT", f, "RIGHT", -8, 0)

	f.manaText:SetJustifyH("LEFT")
	f.manaText:SetWidth(f:GetWidth()*0.7)
	f.manaText:SetPoint("LEFT", f, "LEFT", 8, 0)
end